@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for SMTP Auth
-our $VERSION = '3.20210707'; # VERSION
+our $VERSION = '3.20210825'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 
 sub default_config {
@@ -51,9 +51,13 @@ sub envfrom_callback {
         # Clear the current auth headers ( iprev and helo may already be added )
         # ToDo is this a good idea?
         my $top_handler = $self->get_top_handler();
-        $top_handler->{'c_auth_headers'} = [];
-        $top_handler->{'auth_headers'}   = [];
-        $self->{'is_authenticated'}       = 1;
+        for my $type ( sort keys $top_handler->{'c_auth_headers'}->%* ) {
+            $top_handler->{'c_auth_headers'}->{$type} = [];
+        }
+        for my $type ( sort keys $top_handler->{'auth_headers'}->%* ) {
+            $top_handler->{'auth_headers'}->{$type} = [];
+        }
+        $self->{'is_authenticated'} = 1;
         $self->metric_count( 'authenticated_connect_total' );
         my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'auth' )->safe_set_value( 'pass' );
         $self->add_auth_header( $header );
@@ -79,7 +83,7 @@ Mail::Milter::Authentication::Handler::Auth - Handler class for SMTP Auth
 
 =head1 VERSION
 
-version 3.20210707
+version 3.20210825
 
 =head1 DESCRIPTION
 
