@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for BIMI
-our $VERSION = '3.20210707'; # VERSION
+our $VERSION = '3.20210825'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 use Mail::BIMI 2;
 
@@ -104,7 +104,9 @@ sub eom_callback {
             my $top_handler = $self->get_top_handler();
             my @auth_headers;
             if ( exists( $top_handler->{'auth_headers'} ) ) {
-                @auth_headers = ( @auth_headers, @{ $top_handler->{'auth_headers'} } );
+                for my $type ( sort keys $top_handler->{'auth_headers'}->%* ) {
+                    @auth_headers = ( @auth_headers, @{ $top_handler->{'auth_headers'}->{$type} } );
+                }
             }
             if (@auth_headers) {
                 foreach my $auth_header (@auth_headers) {
@@ -144,7 +146,9 @@ sub eom_callback {
                     my $top_handler = $self->get_top_handler();
                     my @auth_headers;
                     if ( exists( $top_handler->{'auth_headers'} ) ) {
-                        @auth_headers = ( @auth_headers, @{ $top_handler->{'auth_headers'} } );
+                        for my $type ( sort keys $top_handler->{'auth_headers'}->%* ) {
+                            @auth_headers = ( @auth_headers, @{ $top_handler->{'auth_headers'}->{$type} } );
+                        }
                     }
                     if (@auth_headers) {
                         foreach my $auth_header ( @auth_headers ) {
@@ -352,7 +356,7 @@ Mail::Milter::Authentication::Handler::BIMI - Handler class for BIMI
 
 =head1 VERSION
 
-version 3.20210707
+version 3.20210825
 
 =head1 DESCRIPTION
 
